@@ -32,7 +32,7 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
   if (!suppressWarnings(requireNamespace("tcltk"))) {
     stop("The tcltk package is absent. Interactive folder selection cannot function.")
   }
-  
+
   #button handler functions
   onOK <- function() {
     #tcltk::tkmessageBox(message="clicked ok")
@@ -43,26 +43,26 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
     directory <- tcltk::tclvalue(directoryVariable)
     logFileChecked <- as.logical(as.numeric(tcltk::tclvalue(logFileChecked)))
     logFile_TCL <- tcltk::tclvalue(logFile_TCL)
-    
+
     #check date may only be checked if replace outfile is on.
-    if (!replaceOutfileChecked) checkDateChecked <- FALSE
-    
-    if (replaceOutfileChecked && checkDateChecked) replaceOutfileStr <- "modifiedDate"
-    else if (replaceOutfileChecked && !checkDateChecked) replaceOutfileStr <- "always"
-    else if (!replaceOutfileChecked) replaceOutfileStr <- "never"
-    
+    if (isFALSE(replaceOutfileChecked)) checkDateChecked <- FALSE
+
+    if (isTRUE(replaceOutfileChecked) && isTRUE(checkDateChecked)) replaceOutfileStr <- "modifiedDate"
+    else if (isTRUE(replaceOutfileChecked) && isFALSE(checkDateChecked)) replaceOutfileStr <- "always"
+    else if (isFALSE(replaceOutfileChecked)) replaceOutfileStr <- "never"
+
     #if log file unchecked, pass NULL
-    if (!logFileChecked) logFile_TCL <- NULL
-    
+    if (isFALSE(logFileChecked)) logFile_TCL <- NULL
+
 #    cat(paste("directory:", tcltk::tclvalue(directoryVariable), "\nrecurseChecked: ", recursiveChecked,
 #            "\nshowOutputChecked:", showOutputChecked,
 #            "\nreplaceOutfileChecked:", replaceOutfileChecked, "\ncheckDateChecked:", checkDateChecked,
 #            "\nlogFileChecked:", logFileChecked, "\nlogFilename:", as.character(logFile_TCL), "\n"
 #            ))
-    
+
     tcltk::tkgrab.release(top)
     tcltk::tkdestroy(top)
-    
+
     runModels(target=directory, recursive=recursiveChecked,
         showOutput=showOutputChecked, replaceOutfile=replaceOutfileStr, logFile=logFile_TCL)
   }
@@ -74,21 +74,21 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
   onBrowse <- function(){
     #choose.dir is a prettier way to select directory
     #tcltk::tclvalue(directoryVariable) <- tcltk::tclvalue(tcltk::tkchooseDirectory())
-    if (.Platform$OS.type == "unix") runDir <- tcltk::tclvalue(tcltk::tkchooseDirectory())
+    if (isTRUE(.Platform$OS.type == "unix")) runDir <- tcltk::tclvalue(tcltk::tkchooseDirectory())
     else runDir <- tcltk::tclvalue(tcltk::tclVar(utils::choose.dir(tcltk::tclvalue(directoryVariable), "Choose the Mplus Run Directory")))
-    
-    if (!runDir == "NA") {
+
+    if (isFALSE(runDir == "NA")) {
       tcltk::tclvalue(directoryVariable) <- runDir
       #if log file is still at its default (getwd() + "Mplus Run Models.log"), switch to run directory
-      if (userSetLogFile == FALSE) tcltk::tclvalue(logFile_TCL) <- file.path(runDir, "Mplus Run Models.log")
+      if (isFALSE(userSetLogFile)) tcltk::tclvalue(logFile_TCL) <- file.path(runDir, "Mplus Run Models.log")
     }
-    
+
   }
   onLogBrowse <- function(){
     splitPath <- splitFilePath(tcltk::tclvalue(logFile_TCL))
     #logDir <- tcltk::tclvalue(tcltk::tkgetSaveFile(defaultextension="log", initialdir=chartr("/", "\\", splitPath$directory), initialfile=splitPath$filename))
     logDir <- tcltk::tclvalue(tcltk::tkgetSaveFile(defaultextension="log", initialdir=splitPath$directory, initialfile=splitPath$filename))
-    if (!logDir == "") {
+    if (isFALSE(logDir == "")) {
       tcltk::tclvalue(logFile_TCL) <- logDir
       userSetLogFile <<- TRUE #needs to be assigned in parent env
     }
@@ -96,11 +96,11 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
   onReplace <- function() {
     curVal <- as.character(tcltk::tclvalue(replaceOutfileChecked))
     #cat(paste("replace checked is:", curVal))
-    if (curVal == "1") {
+    if (isTRUE(curVal == "1")) {
       tcltk::tkconfigure(checkDateCB, state="!disabled")
       tcltk::tkconfigure(checkDateLabel, state="!disabled")
     }
-    else if (curVal == "0") {
+    else if (isTRUE(curVal == "0")) {
       tcltk::tkconfigure(checkDateCB, state="disabled")
       tcltk::tkconfigure(checkDateLabel, state="disabled")
     }
@@ -123,73 +123,73 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
     }
     #; tcltk::tclvalue(replaceChecked) <- "0" }
   }
-  
+
   top <- tcltk::tktoplevel(borderwidth=10)
   tcltk::tkgrid(tcltk::ttklabel(top, text="Run Mplus Models", font="-size 14 -family Arial -weight bold"), sticky="w")
-  
+
   tcltk::tkwm.title(top, "Run Mplus Models")
   directoryVariable <- tcltk::tclVar(directory)
   locationFrame <- tcltk::ttkframe(top)
   locationField <- tcltk::ttkentry(locationFrame, width="80", textvariable=directoryVariable)
   Browse.but <- tcltk::ttkbutton(locationFrame, text="Browse", command=onBrowse)
-  
+
   tcltk::tkgrid(tcltk::ttklabel(locationFrame,text="Target directory:"), locationField, Browse.but)
   tcltk::tkgrid(locationFrame, sticky="w")
-  
+
   optionsFrame <- tcltk::ttklabelframe(top, text="Options", borderwidth=10, relief="groove")
   #optionsFrame <- tcltk::ttkframe(top, borderwidth=20, relief="groove")
   #tcltk::tkgrid(tcltk::ttklabel(optionsFrame,text="Options", font="-weight bold -size 12"), sticky="w")
   recursiveChecked <- tcltk::tclVar(recursive)
   recurseCB <- tcltk::ttkcheckbutton(optionsFrame, variable=recursiveChecked)
   tcltk::tkgrid(tcltk::ttklabel(optionsFrame,text="Run models in nested subdirectories"), recurseCB, sticky="w")
-  
+
   replaceOutfileChecked <- tcltk::tclVar(replaceOutfile)
   replaceCB <- tcltk::ttkcheckbutton(optionsFrame, variable=replaceOutfileChecked, command=onReplace)
   tcltk::tkgrid(tcltk::ttklabel(optionsFrame,text="Re-run models that have existing output files"), replaceCB, sticky="w")
-  
+
   checkDateChecked <- tcltk::tclVar(checkDate)
-  if (as.logical(as.numeric(replaceOutfile)) == TRUE) initialState <- "!disabled"
+  if (isTRUE(as.logical(as.numeric(replaceOutfile)))) initialState <- "!disabled"
   else initialState <- "disabled"
   checkDateLabel <- tcltk::ttklabel(optionsFrame, text="    -> Only re-run if input file is newer than existing output (check date)   ", state=initialState)
   checkDateCB <- tcltk::ttkcheckbutton(optionsFrame, variable=checkDateChecked, state=initialState)
   tcltk::tkgrid(checkDateLabel, checkDateCB, sticky="w")
-  
+
   showOutputChecked <- tcltk::tclVar(showOutput)
   showOutputCB <- tcltk::ttkcheckbutton(optionsFrame, variable=showOutputChecked)
   tcltk::tkgrid(tcltk::ttklabel(optionsFrame,text="Show Mplus output on console"), showOutputCB, sticky="w")
-  
+
   #include log file checkbox in options frame
   logFileChecked <- tcltk::tclVar(logFile)
   logFileCB <- tcltk::ttkcheckbutton(optionsFrame, variable=logFileChecked, command=onLogcheck)
   tcltk::tkgrid(tcltk::ttklabel(optionsFrame, text="Log Mplus Run Details to File"), logFileCB, sticky="w")
-  
+
   #add the options frame to the dialog
   tcltk::tkgrid(optionsFrame, sticky="w")
-  
+
   #setup the filename field for the log file
   logFile_TCL <- tcltk::tclVar(file.path(directory, "Mplus Run Models.log"))
   userSetLogFile <- FALSE
   logFrame <- tcltk::ttkframe(optionsFrame) #borderwidth="5m")
-  if (as.logical(as.numeric(logFile)) == TRUE) initialState <- "!disabled"
+  if (isTRUE(as.logical(as.numeric(logFile)))) initialState <- "!disabled"
   else initialState <- "disabled"
   logField <- tcltk::ttkentry(logFrame, width="50", textvariable=logFile_TCL, state=initialState)
   logFilenameLabel <- tcltk::ttklabel(logFrame,text="    -> Log filename:", state=initialState)
   BrowseLog.but <- tcltk::ttkbutton(logFrame, text="Browse", command=onLogBrowse, state=initialState)
-  
+
   #add log frame to grid
   tcltk::tkgrid(logFilenameLabel, logField, BrowseLog.but)
   tcltk::tkgrid(logFrame, sticky="w", columnspan="2")
-  
-  
+
+
   #add button frame to grid
   buttonsFrame <- tcltk::ttkframe(top)
   OK.but <- tcltk::ttkbutton(buttonsFrame,text="OK",command=onOK)
   Cancel.but <- tcltk::ttkbutton(buttonsFrame, text="Cancel", command=onCancel)
   tcltk::tkgrid(OK.but, tcltk::ttklabel(buttonsFrame,text="   "), Cancel.but, sticky="w")
   tcltk::tkgrid(buttonsFrame, sticky="w")
-  
+
   #tcltk::tkgrid.size(top)
-  
+
   tcltk::tkwm.resizable(top, 0, 0)
   tcltk::tkbind(top, "<Return>", onOK)
   tcltk::tkbind(top, "<Escape>", onCancel)
@@ -197,8 +197,8 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
   #tcltk::tkgrab.set(top)
   tcltk::tkfocus(top)
   tcltk::tkwait.window(top)
-  
-  
+
+
 }
 
 #' Run Mplus Models
@@ -206,15 +206,15 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
 #' This function runs a group of Mplus models (.inp files) located within a
 #' single directory or nested within subdirectories.
 #'
-#' @param target the directory containing Mplus input files (.inp) to run
-#'   OR the single .inp file to be run. May be a full path, relative path,
+#' @param target a character vector where each element is a directory containing Mplus input files 
+#'   (.inp) to run OR a single .inp file to be run. Elements may be a full path, relative path,
 #'   or a filename within the working directory.
 #'   Defaults to the current working directory. Example: \dQuote{C:/Users/Michael/Mplus Runs}
 #' @param recursive optional. If \code{TRUE}, run all models nested in subdirectories
 #'   within \code{directory}. Defaults to \code{FALSE}. Not relevant if \code{target} is a single file.
 #' @param filefilter a Perl regular expression (PCRE-compatible) specifying particular input
-#'   files to be run within \code{directory}. See \code{regex} or \url{http://www.pcre.org/pcre.txt}
-#'   for details about regular expression syntax. Not relevant if \code{target} is a single file.
+#'   files to be run among those found in \code{target}. See \code{regex} or \url{http://www.pcre.org/pcre.txt}
+#'   for details about regular expression syntax.
 #' @param showOutput optional. If \code{TRUE}, show estimation output (TECH8)
 #'   in the R console. Note that if run within Rgui, output will display within R,
 #'   but if run via Rterm, a separate window will appear during estimation.
@@ -247,112 +247,132 @@ runModels_Interactive <- function(directory=getwd(), recursive="0",
 #'     replaceOutfile="modifiedDate", logFile="MH_RunLog.txt",
 #'     Mplus_command="C:\\Users\\Michael\\Mplus Install\\Mplus51.exe")
 #' }
+#' \dontrun{
+#'   runModels(getwd(), filefilter = "ex8.*", logFile=NULL)
+#' }
 runModels <- function(target=getwd(), recursive=FALSE, filefilter = NULL, showOutput=FALSE,
-    replaceOutfile="always", logFile="Mplus Run Models.log", Mplus_command="Mplus", killOnFail=TRUE, local_tmpdir=FALSE) {
-  
+    replaceOutfile="always", logFile="Mplus Run Models.log", Mplus_command="Mplus", 
+    killOnFail=TRUE, local_tmpdir=FALSE) {
+
   stopifnot(replaceOutfile %in% c("always", "never", "modifiedDate"))
+
+  #helper subfunction to crawl over the target vector, determine if it is a file or folder,
+  #then locate all .inp files, and convert them to absolute paths
+  convert_to_filelist <- function(target, filefilter=NULL, recursive=FALSE) {
+    filelist <- c()
+    for (tt in target) {
+      fi <- file.info(tt)
+      if (isTRUE(is.na(fi$size))) { stop("Cannot find target: ", tt) } #do not tolerate missing files or folders
+        
+      if (isTRUE(fi$isdir)) {
+        #folder
+        ## remove trailing slash, which generates file.exists error on windows: https://bugs.r-project.org/bugzilla/show_bug.cgi?id=14721
+        directory <- sub("(\\\\|/)?$", "", tt, perl=TRUE)
+        ## however, trailing slash is needed if at root of a drive on Windows
+        if (isTRUE(.Platform$OS.type == "windows") && isTRUE(grepl("^[a-zA-Z]:$", directory))) {
+          directory <- paste0(directory, "/")
+        }
+        
+        if (isFALSE(file.exists(directory))) { stop("runModels cannot find directory: ", directory) }
+        
+        #list files in the current directory
+        this_set <- list.files(path=directory, recursive=recursive, pattern=".*\\.inp?$", full.names = TRUE)
+        filelist <- c(filelist, this_set)
+      } else {
+        #element is a file
+        #check file extension
+        if (!grepl(".*\\.inp?$", tt, perl=TRUE)) {
+          warning("Target: ", tt, "does not appear to be an .inp file. Ignoring it.")
+          next
+        } else {
+          if (isFALSE(file.exists(tt))) { stop("runModels cannot find input file: ", tt) }
+          
+          filelist <- c(filelist, tt)
+        }
+      }
+    }
     
-  #TODO: would be good to come back and make this more versatile, supporting a vector target
-  if (length(target) > 1L) { stop("target for runModels must be a single file or single directory")}
+    #apply user filter, if requested
+    if (!is.null(filefilter)) { filelist <- grep(filefilter, filefilter, perl=TRUE, value=TRUE) } 
+    
+    #normalize paths to convert everything to absolute
+    filelist <- normalizePath(filelist)
+    return(filelist)
+  }
+  
+  #Use of ~/ home directory doesn't play well with call to sh later.
+  if (grepl("^~/", Mplus_command, perl=TRUE)) { Mplus_command <- normalizePath(Mplus_command) }
   
   #retain working directory and reset at end of run
   #need to set here to ensure that logTarget initialization below is within target directory, not getwd()
   curdir <- getwd()
-  
-  #check whether target is a directory or a single file
-  if (grepl(".*\\.inp?$", target, perl=TRUE)) { #tolerate .in and .inp files
-    directory <- dirname(target)
-    filelist <- basename(target)
-    
-    if (!is.null(filefilter)) { warning("Using runModels() with a single .inp target ignores filefilter") }
-    if (!file.exists(target)) { stop("runModels cannot locate target file: ", target) }
-    setwd(directory)
-    
-    #look for .out file of the same name to handle skipping existing files downstream.
-    if (file.exists(outtest <- sub("\\.inp?$", ".out", filelist, perl=TRUE))) { filelist <- c(filelist, outtest)}
-    
-  } else {
-    ## remove trailing slash, which generates file.exists error on windows: https://bugs.r-project.org/bugzilla/show_bug.cgi?id=14721
-    directory <- sub("(\\\\|/)?$", "", target, perl=TRUE)
-    ## however, trailing slash is needed if at root of a drive on Windows
-    if (.Platform$OS.type == "windows" && grepl("^[a-zA-Z]:$", directory)) {
-      directory <- paste0(directory, "/")
-    }
-    
-    if (!file.exists(directory)) { stop("runModels cannot change to directory: ", directory) }
-    setwd(directory)
-    
-    #list files in the current directory
-    filelist <- list.files(recursive=recursive, pattern=filefilter)
-  }
-  
+
   normalComplete <- FALSE
-  
+
   #if log file requested, then open file connection for writing
-  if (!is.null(logFile)) {
+  if (isFALSE(is.null(logFile))) {
     logTarget <- file(description = logFile, open = "wt", blocking = TRUE)
     writeLines(c(paste("------Begin Mplus Model Run: ", format(Sys.time(), "%d%b%Y %H:%M:%S"), "------", sep=""),
-            paste("Target directory: ", directory, sep=""),
-            "Run options:",
-            paste("\tRecursive (run models in subdirectories):", as.character(recursive)),
-            paste("\tShow output on console:", as.character(showOutput)),
-            paste("\tReplace existing outfile:", replaceOutfile),
-            "------"
-        ), con=logTarget)
+                 "Run options:",
+                 paste("\tRecursive (run models in subdirectories):", as.character(recursive)),
+                 paste("\tShow output on console:", as.character(showOutput)),
+                 paste("\tReplace existing outfile:", replaceOutfile),
+                 "------"
+    ), con=logTarget)
     #need to flush after each writeLines to keep the text file current.
     flush(logTarget)
   }
   
   isLogOpen <- function() {
     #if null is passed as the log file, then it is by definition not open (non-existent)
-    if (is.null(logFile)) return(FALSE)
-    
+    if (isTRUE(is.null(logFile))) return(FALSE)
+
     connections <- data.frame(showConnections(all = FALSE))
-    if (length(grep(splitFilePath(logFile)$filename, connections$description, ignore.case=TRUE)) > 0) return(TRUE)
+    if (isTRUE(length(grep(splitFilePath(logFile)$filename, connections$description, ignore.case=TRUE)) > 0)) return(TRUE)
     else return(FALSE)
   }
-  
+
   #if the function gets interrupted (e.g., the user presses escape), kill the Mplus process (doesn't happen automatically).
   exitRun <- function() {
     deleteOnKill <- TRUE #whether to delete unfinished output
-    
-    if (normalComplete == FALSE && isLogOpen()) {
+
+    if (isFALSE(normalComplete) && isTRUE(isLogOpen())) {
       writeLines("Run terminated abnormally", logTarget)
       flush(logTarget)
     }
-    
+
     #create a data frame consisting of the process names and pids
     #uses str split on the output of wmic to extract name and pid columns
     #depends on windows tools
-    if (.Platform$OS.type == "windows" && normalComplete == FALSE && killOnFail == TRUE) {
+    if (isTRUE(.Platform$OS.type == "windows") && isFALSE(normalComplete) && isTRUE(killOnFail)) {
       processList <- ldply(strsplit(shell("wmic process get caption, processid", intern=TRUE), split="\\s+", perl=TRUE),
           function(element) {
             return(data.frame(procname=element[1], pid=element[2], stringsAsFactors = FALSE))
           })
-      
-      if(length(grep("mplus.exe", processList$procname, ignore.case=TRUE)) > 0) {
-        if(isLogOpen()) {
+
+      if(isTRUE(length(grep("mplus.exe", processList$procname, ignore.case=TRUE)) > 0)) {
+        if(isTRUE(isLogOpen())) {
           writeLines("Killing wayward Mplus processes", logTarget)
           flush(logTarget)
         }
         shell("taskkill /f /im mplus.exe")
-        
+
         #if the process is currently running and we kill it, then the output and gph files will be incomplete.
         #in general, it would be good to delete these.
-        if(deleteOnKill == TRUE) {
-          noExtension <- substr(absFilename, length(absFilename) - 4, length(absFilename))
+        if(isTRUE(deleteOnKill)) {
+          noExtension <- substr(cur_inpfile, length(cur_inpfile) - 4, length(cur_inpfile))
           outDelete <- paste(noExtension, ".out", sep="")
           gphDelete <- paste(noExtension, ".gph", sep="")
-          if (file.exists(outDelete)) {
+          if (isTRUE(file.exists(outDelete))) {
             unlink(outDelete)
-            if(isLogOpen()) {
+            if(isTRUE(isLogOpen())) {
               writeLines(paste("Deleting unfinished output file:", outDelete), logTarget)
               flush(logTarget)
             }
           }
-          if (file.exists(gphDelete)) {
+          if (isTRUE(file.exists(gphDelete))) {
             unlink(gphDelete)
-            if(isLogOpen()) {
+            if(isTRUE(isLogOpen())) {
               writeLines(paste("Deleting unfinished graph file:", gphDelete), logTarget)
               flush(logTarget)
             }
@@ -360,52 +380,60 @@ runModels <- function(target=getwd(), recursive=FALSE, filefilter = NULL, showOu
         }
       }
     }
-    
+
     #close logfile
-    if (isLogOpen()) { close(logTarget) }
-    
+    if (isTRUE(isLogOpen())) { close(logTarget) }
+
     #reset working directory
     setwd(curdir)
   } #end exitRun definition
-  
+
   on.exit(exitRun())
-    
-  #select only .inp files using grep
-  inpfiles <- filelist[grep(".*\\.inp?$", filelist, ignore.case=TRUE)] #tolerate .in and .inp files
-  outfiles <- filelist[grep(".*\\.out$", filelist, ignore.case=TRUE)]
-  
-  if(length(inpfiles) < 1) stop("No Mplus input files detected in the target directory: ", directory)
+
+  #find all inp files in each element of target and return a single vector of absolute paths to inp files
+  inpfiles <- convert_to_filelist(target, filefilter, recursive)
+
+  #find .out files corresponding to each .inp file  
+  outfiles <- unlist(lapply(inpfiles, function(x) {
+    if (file.exists(otest <- sub("\\.inp?$", ".out", x))) {
+      return(otest)
+    } else {
+      return(NULL)
+    }
+  }))
   
+  if(isTRUE(length(inpfiles) < 1)) stop("No Mplus input files detected in the provided target: ", paste(target, collapse=", "))
+
   dropOutExtensions <- sapply(outfiles, function(x) {
-        if (nchar(x) >= 4) return(tolower(substr(x, 1, (nchar(x)-4))))
-      })
+    if (isTRUE(nchar(x) >= 4)) return(tolower(substr(x, 1, (nchar(x)-4))))
+  })
   
   for (i in 1:length(inpfiles)) {
-    
-    if (!replaceOutfile == "always") {
+
+    if (isFALSE(replaceOutfile == "always")) {
       #if there is a match in the outfiles for this input file, then decide whether to skip
-      if (tolower(sub("\\.inp?$", "", inpfiles[i], perl=TRUE)) %in% dropOutExtensions) {
-        
-        if (replaceOutfile == "modifiedDate") {
+      if (isTRUE(tolower(sub("\\.inp?$", "", inpfiles[i], perl=TRUE)) %in% dropOutExtensions)) {
+
+        if (isTRUE(replaceOutfile == "modifiedDate")) {
           #if check date is true, then the output file must exist and it must be
           #older than the input file to re-run
           inpmtime <- file.info(inpfiles[i])$mtime
-          
+
           #need to locate the exact outfile match
           matchPos <- grep(tolower(substr(inpfiles[i], 1, (nchar(inpfiles[i]) - 4))), dropOutExtensions)
-          if (length(matchPos) < 1) warning("Could not locate matching outfile")
+          if (isTRUE(length(matchPos) < 1)) warning("Could not locate matching outfile")
           outmtime <- file.info(outfiles[matchPos[1]])$mtime
-          
-          if (inpmtime <= outmtime) {
-            if (isLogOpen()) {
+
+          if (isTRUE(inpmtime <= outmtime)) {
+            if (isTRUE(isLogOpen())) {
               writeLines(paste("Skipping model because output file is newer than input file:", inpfiles[i]), logTarget)
               flush(logTarget)
             }
             next
           }
         }
-        else if (replaceOutfile == "never"){
-          if (isLogOpen()) {
+        else if (isTRUE(replaceOutfile == "never")) {
+          if (isTRUE(isLogOpen())) {
             writeLines(paste("Skipping model because output file already exists for:", inpfiles[i]), logTarget)
             flush(logTarget)
           }
@@ -413,78 +441,71 @@ runModels <- function(target=getwd(), recursive=FALSE, filefilter = NULL, showOu
         }
       }
     }
-    
+
     #split input file into one element per directory (e.g., "dir1/dir2/mytest.inp" becomes a 3-element vector
-    #code adapted from R.utils filePath command
-    inputSplit <- splitFilePath(inpfiles[i])
-    if (is.na(inputSplit$directory)) dirtocd <- directory
-    else dirtocd <- file.path(directory, inputSplit$directory)
-    
-    #the absolute path to the file combines the passed-in directory with the subdirectories
-    #identified in the case of recursive=T
-    
-    absFilename <- file.path(directory, inpfiles[i])
-    
+    inputSplit <- splitFilePath(inpfiles[i], normalize=TRUE)
+
+    cur_inpfile <- inpfiles[i] #tmp var used in case of attempt to kill current job
+
     #UPDATE 21Oct2011: Since mplus has been released for linux, don't default to wine.
-    if (.Platform$OS.type == "unix" && Mplus_command == "Mplus") {
-      if (Sys.info()["sysname"] == "Darwin") Mplus_command <- "/Applications/Mplus/mplus"
+    if (isTRUE(.Platform$OS.type == "unix") && isTRUE(Mplus_command == "Mplus")) {
+      if (isTRUE(Sys.info()["sysname"] == "Darwin")) Mplus_command <- "/Applications/Mplus/mplus"
       else Mplus_command <- "mplus" #linux is case sensitive
     }
-    
+
     #navigate to working directory in DOS using cd command so that Mplus finds the appropriate files (support rel paths)
     #switched over to use relative filename because of problems in Mplus via Wine misinterpreting absolute paths due to forward slashes.
     #25Jul2012: Quote Mplus_command in case it's in a path with spaces.
-    command <- paste("cd \"", dirtocd, "\" && \"", Mplus_command, "\" \"", inputSplit$filename, "\"", sep="")
-    
+    command <- paste("cd \"", inputSplit$directory, "\" && \"", Mplus_command, "\" \"", inputSplit$filename, "\"", sep="")
+
     #allow for divergence if the package is being run in Linux (Mplus via wine)
-    if (.Platform$OS.type == "windows") {
+    if (isTRUE(.Platform$OS.type == "windows")) {
       #Given that Mplus is a Windows program, should generally automatically remap / as \ for Windows
       #remap forward slashes to backslashes
       command <- chartr("/", "\\", command)
-      
+
       #code swiped from shell because shell didn't support suppressing output
       shellcommand <- Sys.getenv("COMSPEC")
       flag <- "/c"
-      
+
       #assemble full command from the shell call, flag, and command
       command <- paste(shellcommand, flag, command)
-    }
-    else if (.Platform$OS.type == "unix") {
+    } else if (isTRUE(.Platform$OS.type == "unix")) {
       #allow for unix-specific customization here
     }
-    
-    if (isLogOpen()) {
+
+    if (isTRUE(isLogOpen())) {
       writeLines(paste("Currently running model:", inputSplit$filename), logTarget)
       flush(logTarget)
     }
-    
+
     #run the model
     cat("\nRunning model:", inputSplit$filename, "\n")
     cat("System command:", command, "\n")
-    
+
     #unix system command does not have show.output.on.console or invisible parameters
-    if (.Platform$OS.type == "windows")	{
+    if (isTRUE(.Platform$OS.type == "windows"))	{
       system(command, show.output.on.console = showOutput, invisible=(!showOutput), wait=TRUE)
     } else {
-      if(showOutput) stdout.value = ""
+      if(isTRUE(showOutput)) stdout.value = ""
       else stdout.value = NULL
       #need to switch to each directory, then run Mplus within using just the filename
       oldwd <- getwd()
-      setwd(dirtocd)
-      if (local_tmpdir) { Sys.setenv(TMPDIR=dirtocd) } #define TMPDIR local to the .inp file to execute
+      setwd(inputSplit$directory)
+      if (isTRUE(local_tmpdir)) { Sys.setenv(TMPDIR=inputSplit$directory) } #define TMPDIR local to the .inp file to execute
       exitCode <- system2(Mplus_command, args=c(shQuote(inputSplit$filename)), stdout=stdout.value, wait=TRUE)
-      if (exitCode > 0L) {
+      if (isTRUE(exitCode > 0L)) {
         warning("Mplus returned error code: ", exitCode, ", for model: ", inputSplit$filename, "\n")
       }
       setwd(oldwd)
     }
   }
-  
-  if (isLogOpen()) {
+
+  if (isTRUE(isLogOpen())) {
     writeLines(c("", paste("------End Mplus Model Run: ", format(Sys.time(), "%d%b%Y %H:%M:%S"), "------", sep="")), logTarget)
     flush(logTarget)
   }
   normalComplete <- TRUE #if we made it here, then all completed normally
-  
+
   #exitRun will fire here if all successful.
 }
